import React from 'react'
import {
  Layout,
  Menu,
  Breadcrumb,
  Avatar,
  Button,
  Row,
  Col,
  Dropdown,
  Tooltip,
  Typography,
  Grid,
} from 'antd'
import {
  HomeOutlined,
  LoginOutlined,
  LogoutOutlined,
  MenuFoldOutlined,
  MenuUnfoldOutlined,
  SettingOutlined,
  UserOutlined,
} from '@ant-design/icons'
import { useNavigate, useLocation, Link } from 'react-router-dom'
import { useAuth } from '../../hooks'
import { dashboardRoutes } from '../../routers'
import { Helmet } from 'react-helmet-async'
import './DashboardLayout.css'

const { Header, Content, Sider, Footer } = Layout
const rootSubmenuKeys = ['']

const nombrePagina = import.meta.env.VITE_NOMBRE_PAGINA
const version = import.meta.env.VITE_API_VERSION

const { useBreakpoint } = Grid

const DashboardLayout = ({ children }) => {
  // const themeMode = "dark";
  const navigate = useNavigate()
  const location = useLocation()
  const { userLoading, signOut, session, user } = useAuth()

  const [collapsed, setCollapsed] = React.useState(false)
  const [openKeys, setOpenKeys] = React.useState([''])
  const [selectedKey, setSelectedKey] = React.useState('')
  const [breadcrumbItems, setBreadcrumbItems] = React.useState([])
  const [titulo, setTitulo] = React.useState('')

  const pantalla = useBreakpoint()

  const dashStyles = {
    logoCollapsed: {
      height: 48,
      margin: 6,
      backgroundSize: 'contain',
      backgroundPosition: 'center center',
      transition: 'opacity 1s ease-in-out',
      backgroundImage: `url("/logoCollapsed.png")`,
      backgroundRepeat: 'no-repeat',
    },
    logo: {
      height: 120,
      margin: 20,
      backgroundSize: 'contain',
      backgroundRepeat: 'no-repeat',
      backgroundPosition: 'center center',
      transition: 'opacity 1s ease-in-out',
      backgroundImage: `url("/logo.png")`,
    },
    header: {
      padding: 0,
      backgroundColor: '#fff',
    },
    trigger: {
      color: '#333',
      paddingLeft: 10,
    },
    breadcrumb: {
      display: 'flex',
      alignItems: 'center',
      marginLeft: 5,
      height: '100%',
    },
    user: {
      display: 'flex',
      justifyContent: 'space-evenly',
      alignItems: 'center',
      paddingRight: 10,
    },
    footer: {
      textAlign: 'center',
    },
  }

  const items = [
    {
      key: '1',
      label: <Link to="/perfil">Configuración del perfil</Link>,
      icon: <SettingOutlined/>,
    },
    {
      key: '2',
      danger: true,
      label: 'Cerrar sesión',
      icon: <LoginOutlined/>,
      onClick: () => signOut(),
    },
  ]

  const onOpenChange = (keys) => {
    const latestOpenKey = keys.find((key) => openKeys.indexOf(key) === -1)
    if (rootSubmenuKeys.indexOf(latestOpenKey) === -1) {
      setOpenKeys(keys)
    } else {
      setOpenKeys(latestOpenKey ? [latestOpenKey] : [])
    }
  }

  const sidebarMapper = (route) => {
    let puedeVer = user?.permisos.find((permiso) => permiso === route?.ver)
    // if (puedeVer) {
      if (route.sidebar === 'single') {
        return {
          key: route.path,
          icon: route.icon,
          label: route.name,
          onClick: () => {
            setSelectedKey(route.path)
            navigate(route.path)
          },
        }
      } else if (route.sidebar === 'collapse') {
        const innerMap = (r) => ({ ...r, path: route.path + r.path })
        const finalKey = 'collapse-' + route.path
        return {
          key: finalKey,
          icon: route.icon,
          label: route.name,
          children: route.routes.map(innerMap).map(sidebarMapper),
        }
      }
    // }
    return null
  }

  React.useEffect(() => {
    const rutasBreadCrumbs = (
      rutasOrig,
      rutaDividida,
      indice = 0,
      ruta = ''
    ) => {
      let rutas = []
      let path = ''
      if (indice === 0) {
        return rutasBreadCrumbs(rutasOrig, rutaDividida, indice + 1)
      }
      if (indice > rutaDividida.length - 1) {
        return rutas
      }
      if (rutaDividida.length >= indice + 1 && rutaDividida[indice] !== '') {
        path = rutasOrig?.find(
          (r) => r?.path?.indexOf(rutaDividida[indice]) !== -1
        )
        if (path !== undefined) {
          ruta += path?.path
          rutas.push({
            name: path?.name,
            to: ruta,
            icon: path?.icon,
          })
        }
        rutas = [
          ...rutas,
          ...rutasBreadCrumbs(path?.routes, rutaDividida, indice + 1, ruta),
        ]
      }
      return rutas
    }

    let rutas = [
      { name: 'Inicio', to: '/', icon: <HomeOutlined/> },
      ...rutasBreadCrumbs(dashboardRoutes, location?.pathname?.split('/')),
    ]

    setTitulo(rutas[rutas?.length - 1]?.name)
    setBreadcrumbItems(rutas)
  }, [location?.pathname])

  React.useEffect(() => {
    const flatter = (r) =>
      r?.routes
        ? [
          r,
          ...r?.routes
            .map((sub) => ({ ...sub, path: r.path + sub.path }))
            .flatMap(flatter),
        ]
        : r
    const flatted = dashboardRoutes.flatMap(flatter)
    const paths = flatted.map((r) => r.path)
    const key = paths.find((path) => path === location?.pathname)
    setSelectedKey(key)
    const tmpOpenKeys = flatted
      .filter(
        (r) => r.sidebar === 'collapse' && location?.pathname.includes(r.path)
      )
      .map((r) => 'collapse-' + r.path)
    setOpenKeys(tmpOpenKeys)
  }, [location])

  if (!session && userLoading) return null

  return (
    <Layout style={{ minHeight: '100vh' }}>
      <Helmet>
        <title>
          {nombrePagina} - {titulo || ''}{' '}
        </title>
      </Helmet>
      <Sider
        trigger={null}
        collapsible
        theme="light"
        collapsed={collapsed}
        width={250}
      >
        {/* AQUÍ VA EL LOGO */}
        <div /* style={collapsed ? dashStyles.logoCollapsed : dashStyles.logo} *//>
        <Menu
          theme="light"
          mode="inline"
          openKeys={openKeys}
          onOpenChange={onOpenChange}
          selectedKeys={selectedKey}
          items={[
            ...dashboardRoutes.map(sidebarMapper),
            {
              key: 'logout',
              icon: <LogoutOutlined/>,
              label: 'Cerrar sesión',
              onClick: () => signOut(),
            }
          ]}
          className="layout-menu"
        />
      </Sider>
      <Layout className="site-layout">
        <Header
          className="site-layout-background"
          style={dashStyles.header}
          theme="light"
        >
          <Row>
            <Col span={4} sm={2} xxl={1} style={{ textAlign: 'center' }}>
              <Button
                type="link"
                icon={collapsed ? <MenuUnfoldOutlined/> : <MenuFoldOutlined/>}
                style={dashStyles.trigger}
                onClick={() => setCollapsed(!collapsed)}
              />
            </Col>
            <Col xs={20} sm={15} md={16} lg={16} xxl={19}>
              <Breadcrumb
                style={dashStyles.breadcrumb}
                items={breadcrumbItems?.map((item, index) => ({
                  title: (
                    <Link to={item?.to}>
                      {item?.icon}
                      <span> {item?.name} </span>
                    </Link>
                  ),
                }))}
              />
            </Col>
            <Col xs={24} sm={7} md={6} lg={6} xxl={4} style={dashStyles.user}>
              <Tooltip title={user?.correo}>
                <div style={{ display: 'flex', flexDirection: 'column' }}>
                  <Typography.Text strong style={{ textAlign: 'end' }}>
                    {user?.nombre}
                  </Typography.Text>
                  {pantalla?.xl ? (
                    <Typography.Text type="secondary">
                      {user?.correo}
                    </Typography.Text>
                  ) : (
                    ''
                  )}
                </div>
              </Tooltip>
              <Dropdown menu={{ items }}>
                <Avatar shape="square" size={user?.foto ? 60 : 40} icon={
                  <UserOutlined/>} src={user?.foto}/>
              </Dropdown>
            </Col>
          </Row>
        </Header>
        <Content
          className="site-layout-background"
          style={{
            margin: 10,
            minHeight: 280,
          }}
          children={children}
        />
        <br/>
        <Footer style={dashStyles.footer}>
          {version} - &copy; Derechos reservados.
        </Footer>
      </Layout>
    </Layout>
  )
}

export default DashboardLayout
